local HerbsAndOreTest = {}
HerbsAndOreTest.__index = HerbsAndOreTest

setmetatable(HerbsAndOreTest, {
  __index = UnitTest, -- this is what makes the inheritance work
  __call = function (cls)
    local self = setmetatable({}, cls)
    UnitTest:_init(self)
    return self
  end
})

local ItemsTest = {}
ItemsTest.__index = ItemsTest

setmetatable(ItemsTest, {
  __index = UnitTest, -- this is what makes the inheritance work
  __call = function (cls)
    local self = setmetatable({}, cls)
    UnitTest:_init(self)
    return self
  end
})


function HerbsAndOreTest.test_requiredStackAmountForMillingProspecting(self)
    self:assertEquals(DestroyusPro.isStackAmountEnough(1), 
	                  false, "Stack 1 should not be enough")
    self:assertEquals(DestroyusPro.isStackAmountEnough(5), 
	                  true, "Stack 5 should be enough")
    self:assertEquals(DestroyusPro.isStackAmountEnough(4),
	                  false, "Stack 4 should not be enough")
    self:assertEquals(DestroyusPro.isStackAmountEnough(6),
	                  true, "Stack 6 should be enough")
end

function HerbsAndOreTest.NOtest_isHerb(self)
    self:assertEquals(DestroyusPro.isHerb("Trade Goods", "Herb"), true)
    self:assertEquals(DestroyusPro.isHerb("Trade Goods", "Metal & Stone"), false)
    self:assertEquals(DestroyusPro.isHerb("Weapons", "Herb"), false)
end

function HerbsAndOreTest.NOtest_isOre(self)
    self:assertEquals(DestroyusPro.isOre("Trade Goods", "Metal & Stone",
                                         2770), true)
    self:assertEquals(DestroyusPro.isOre("Trade Goods", "Metal & Stone",
                                         2840), false)
    self:assertEquals(DestroyusPro.isOre("Trade Goods", "Metal & Stone",
                                         72092), true)
    self:assertEquals(DestroyusPro.isOre("Weapons", "Metal & Stone",
                                         23425), false)
end

function HerbsAndOreTest.test_herbQualityAllowed(self)
    self:assertEquals(DestroyusPro.isQualityHerbAllowed(1), 
	                  true, "Quality 1 should be allowed")
    self:assertEquals(DestroyusPro.isQualityHerbAllowed(2),
	                  true, "Quality 2 should be allowed")
    self:assertEquals(DestroyusPro.isQualityHerbAllowed(0),
	                  false, "Quality 0 should not be allowed")
    self:assertEquals(DestroyusPro.isQualityHerbAllowed(3),
	                  false, "Quality 3 should be allowed")
end

function HerbsAndOreTest.test_isItemNotConsumable(self)
    self:assertEquals(DestroyusPro.isItemNotConsumable(109624), 
	                  false, "Items should not be consumable case 1") 
    self:assertEquals(DestroyusPro.isItemNotConsumable(109125),
	                  true, "Items should be consumable case 2")
    self:assertEquals(DestroyusPro.isItemNotConsumable(108353),
	                  false, "Items should not be consumable case 3")
    self:assertEquals(DestroyusPro.isItemNotConsumable(79010),
	                  true, "Items should be consumable case 4")
end

function HerbsAndOreTest.test_haveItemRequiredProfLvl(self)
	-- Gromsblood
    self:assertEquals(DestroyusPro.haveItemRequiredProfLvl(8846,
        773, DestroyusPro.herbsIlvl), true, "Gromsblood should have required prof lvl")
	-- Snow Lily
    self:assertEquals(DestroyusPro.haveItemRequiredProfLvl(79010,
        773, DestroyusPro.herbsIlvl), false, "Snow Lily should not have required prof lvl") 
	-- Fireweed
    self:assertEquals(DestroyusPro.haveItemRequiredProfLvl(109125,
        773, DestroyusPro.herbsIlvl), true, "Fireweed should have required prof lvl")
	-- Starlight Rose
    self:assertEquals(DestroyusPro.haveItemRequiredProfLvl(124105,
        773, DestroyusPro.herbsIlvl), true, "Starlight should have required prof lvl")
    self:assertEquals(DestroyusPro.haveItemRequiredProfLvl(55290,
        773, DestroyusPro.herbsIlvl), false, "Razorwind Broadaxe(Axe) should not have required prof lvl")		
end

function HerbsAndOreTest.test_isNotUniqueItem(self)
	self:assertEquals(DestroyusPro.isNotUniqueItem(8153, DestroyusPro.itemsIdNotForMilling), 
	                  false, "Items should not be unique!")
	self:assertEquals(DestroyusPro.isNotUniqueItem(79010, DestroyusPro.itemsIdNotForMilling),
	                  true, "Items should be unique!")
end

function HerbsAndOreTest.test_isItemNotSellable(self)
	-- Item is not for sell 
	self:assertEquals(DestroyusPro.isItemNotSellable(true),
	                  false, "Item should be unique and not for sell")
	-- Item is for sell
	self:assertEquals(DestroyusPro.isItemNotSellable(false),
	                  true, "Item should be unique and can be sold")
end

function HerbsAndOreTest.test_isItemMillingable(self)
	-- Herb which can not be millingable
	self:assertEquals(DestroyusPro.isItemMillingable(79010, 5, false),
	                  false, "Herb can not be millingable")
	-- Stack size good and herb is millingable
	self:assertEquals(DestroyusPro.isItemMillingable(109125, 10, false),
	                  true, "Stack size bad or herb is not millingable")
	-- Stack size small when herb is millingable
	self:assertEquals(DestroyusPro.isItemMillingable(109125, 4, false),
	                  false, "Stack size should be 4 < 5!")  
	-- Lvl of prof skill is better than herb 
	self:assertEquals(DestroyusPro.isItemMillingable(8846, 10, false),
	                  true, "Skill lvl is lover than herb lvl")
	-- Low lvl skill 475 > proff lvl
	self:assertEquals(DestroyusPro.isItemMillingable(52983, 5, false),
	                  false, "Herb can not be milled in current prof lvl")
	-- Item is not herb
	self:assertEquals(DestroyusPro.isItemMillingable(55290, 5, false),
	                  false, "Item is not a herb!")
end

function HerbsAndOreTest.test_isItemProspectingable(self)
	-- Copper Ore
	self:assertEquals(DestroyusPro.isItemProspectingable(2770, 5, false),
	                  true, "Copper ore should be Prospectable")
	-- Iron Ore and prof lvl are equals
	self:assertEquals(DestroyusPro.isItemProspectingable(2772, 5, false),
	                  true, "Iron Ore and prof lvl should be equals") 
	-- Iron ore and prof lvl are equals, stack small
	self:assertEquals(DestroyusPro.isItemProspectingable(2772, 4, false),
	                  false, "ron ore and prof lvl are equals, stack 4 < 5!") 
	-- Fel Iron Ore prof lvl to small
	self:assertEquals(DestroyusPro.isItemProspectingable(23424, 5, false),
	                 false, "Fel Iron Ore should not have been noticed") 
	-- Ghost Iron Ore lvl to small
	self:assertEquals(DestroyusPro.isItemProspectingable(72092, 5, false),
	                 false, "Ghost Iron Ore should not have been noticed")
	-- Copper Bar
	self:assertEquals(DestroyusPro.isItemProspectingable(2840, 50, false),
	                  false, "Copper ore is not Prospectingable")
end

function HerbsAndOreTest.setUp(self)
	self.functionProfInfo = GetProfessionInfo
	GetProfessionInfo = GetProfessionInfoApi
	self.functionGetProf = GetProfessions
	GetProfessions = function() return 1,2,3,4,5,6,7 end
end

function HerbsAndOreTest.tearDown(self)
	GetProfessionInfo = self.functionProfInfo
	GetProfessions = self.functionGetProf
end

function ItemsTest.test_isItem(self)
	-- Sunspring Vest 106719
	local item = DestroyusPro.createItemInfoTab(106719)
	self:assertEquals(DestroyusPro.isItem(item["class"], item["subclass"], item["equipslot"]),
	                  true, "Sunspring Vest should be seen as item!")
end

function ItemsTest.test_isWeapon(self)
	-- Sunspring Vest 106719
	local item = DestroyusPro.createItemInfoTab(106719)
	self:assertEquals(DestroyusPro.isWeapon(item["class"], item["subclass"]),
	                  false, "Sunspring Vest is a Armor!")
	-- Tracker's Musket of the Bandit 36643 
	local item = DestroyusPro.createItemInfoTab(36643)
	self:assertEquals(DestroyusPro.isWeapon(item["class"], item["subclass"]),
	                  true, "Tracker's Musket of the Bandit should be Weapon!")
	-- Oval Ring of Stamina 36417 
	local item = DestroyusPro.createItemInfoTab(36417)
	self:assertEquals(DestroyusPro.isWeapon(item["class"], item["subclass"]),
	                  false, "Oval Ring of Stamina is a Miscellaneous!")

end

function ItemsTest.test_isArmor(self)
	-- Sunspring Vest 106719
	local item = DestroyusPro.createItemInfoTab(106719)
	self:assertEquals(DestroyusPro.isArmor(item["class"], item["subclass"]),
	                  true, "Sunspring Vest should be armor!")
	-- Tracker's Musket of the Bandit 36643 
	local item = DestroyusPro.createItemInfoTab(36643)
	self:assertEquals(DestroyusPro.isArmor(item["class"], item["subclass"]),
	                  false, "Tracker's Musket of the Bandit is Weapon!") 
	-- Oval Ring of Stamina 36417 
	local item = DestroyusPro.createItemInfoTab(36417)
	self:assertEquals(DestroyusPro.isArmor(item["class"], item["subclass"]),
	                  false, "Oval Ring of Stamina is a Miscellaneous!") 
	
end 

function ItemsTest.test_isMiscellaneous(self)
	-- Sunspring Vest 106719
	local item = DestroyusPro.createItemInfoTab(106719)
	self:assertEquals(DestroyusPro.isMiscellaneous(item["class"], item["subclass"], item["equipslot"]),
	                  false, "Sunspring Vest is armor!")
	-- Tracker's Musket of the Bandit 36643 
	local item = DestroyusPro.createItemInfoTab(36643)
	self:assertEquals(DestroyusPro.isMiscellaneous(item["class"], item["subclass"], item["equipslot"]),
	                 false, "Tracker's Musket of the Bandit is Weapon!")
	-- Oval Ring of Stamina 36417 
	local item = DestroyusPro.createItemInfoTab(36417)
	self:assertEquals(DestroyusPro.isMiscellaneous(item["class"], item["subclass"], item["equipslot"]),
	                  true, "Oval Ring of Stamina should be a Miscellaneous!")
end 

function ItemsTest.test_getItemProffLvlToDisArmor(self)
	-- Sunspring Vest 106719
	local item = DestroyusPro.createItemInfoTab(106719)
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(item["ilevel"], item["quality"]),
	                 1, "Proff lvl is different than 1!")
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(nil, 3),
	                  DestroyusPro.numberNotExist, "Value should not exist! ")
	-- Holy Shroud 2721 rare
	local item = DestroyusPro.createItemInfoTab(2721)
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(item["ilevel"], item["quality"]),
	                 75, "Proff lvl is different than 75!")
	-- Belt of the Blood Nova 50015 epic
	local item = DestroyusPro.createItemInfoTab(50015)
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(item["ilevel"], item["quality"]),
	                  375, "Proff lvl is different than 375!") 	
end

function ItemsTest.test_getItemProffLvlToDisWeapon(self)
	-- Tracker's Musket of the Bandit 36643 
	local item = DestroyusPro.createItemInfoTab(36643)
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(item["ilevel"], item["quality"]),
	                  325, "Value of weapon is different tan 325!")
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(-1, 3),
	                  DestroyusPro.numberNotExist, "Number should not exist. Case 1")
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(-1, 4),
	                  DestroyusPro.numberNotExist, "Number should not exist. Case 2")
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(-1, 2),
	                  DestroyusPro.numberNotExist, "Number should not exist. Case 3")
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(1, -1), 
	                  DestroyusPro.numberNotExist, "Number should not exist. Case 4")
end

function ItemsTest.test_getItemProffLvlToDisRing(self)
	-- Oval Ring of Stamina 36417 
	local item = DestroyusPro.createItemInfoTab(36417)
	self:assertEquals(DestroyusPro.getItemProffLvlToDis(item["ilevel"], item["quality"]),
	                  325, "Value is different than 325!")
end

function ItemsTest.test_isItemEquippable(self)
	-- Oval Ring of Stamina 36417 
	self:assertEquals(DestroyusPro.isItemEquippable(36417),
	                  true, "Ring can not be Equippable")
	-- Sunspring Vest 106719
	self:assertEquals(DestroyusPro.isItemEquippable(106719),
	                  true, "Chest can not be Equippable") 
	-- Gromsblood 8846
	self:assertEquals(DestroyusPro.isItemEquippable(8846),
	                  false, "This item should not be Equippable")
end

function ItemsTest.test_isOnlyOneMaxStack(self)
	-- Tracker's Musket of the Bandit 36643 
	local item = DestroyusPro.createItemInfoTab(36643)
	self:assertEquals(DestroyusPro.isOnlyOneMaxStack(item["maxstack"]),
	                  true, "Item do not have one stack")
	-- Gromsblood 8846
	local item = DestroyusPro.createItemInfoTab(8846)
	self:assertEquals(DestroyusPro.isOnlyOneMaxStack(item["maxstack"]),
	                 false, "Item should not be only one stack!")
end

function ItemsTest.test_isQualityAllowed(self)
	-- Tracker's Musket of the Bandit 36643 
	local item = DestroyusPro.createItemInfoTab(36643)
	self:assertEquals(DestroyusPro.isQualityAllowed(item["quality"]),
	                  true, "Quality should be allowed!")
end

function ItemsTest.test_isItemDisenchantable(self) 
	local functionGetProfEnch = GetProfessions
	GetProfessions = function() return 2,3,1,4,5,6,7 end
	-- Sunspring Vest 106719
	self:assertEquals(DestroyusPro.isItemDisenchantable(106719, 1, false),
	                  true, "Sunspring Vest 106719")
	-- Scorching Cloak 134634
	self:assertEquals(DestroyusPro.isItemDisenchantable(134634, 1, false),
	                  false, "Scorching Cloak 134634")
	-- Monastic Ring of Stoicism  88638  UniqueId
	self:assertEquals(DestroyusPro.isItemDisenchantable(88638, 1, false),
	                  false, "Monastic Ring of Stoicism  88638  UniqueId")
	-- Necropolis Breastplate 134729
	self:assertEquals(DestroyusPro.isItemDisenchantable(36397, 1, false),
	                  true, "Necropolis Breastplate 134729")
	-- Belt of the Blood Nova 50015 Quality epic 
	self:assertEquals(DestroyusPro.isItemDisenchantable(50015, 1, false),
	                  false, "Belt of the Blood Nova 50015 Quality epic") 
	-- Jacinth Circle 11969
	self:assertEquals(DestroyusPro.isItemDisenchantable(11969, 1, false),
	                  true, "Jacinth Circle 11969")
	-- Hypnotist's Watch 25786 Trinket 225 ILVL
	self:assertEquals(DestroyusPro.isItemDisenchantable(25786, 1, false),
					  true, "Hypnotist's Watch 25786 Trinket 225 ILVL")
	GetProfessions = functionGetProfEnch
	
end

function ItemsTest.test_isItemDisenchantableLowLvlSkill(self) 
	local functionGetProfEnch = GetProfessions
	GetProfessions = function() return 2,4,1,4,5,6,7 end
	-- Sunspring Vest 106719
	self:assertEquals(DestroyusPro.isItemDisenchantable(106719, 1, false),
	                  true, "Sunspring Vest 106719")
	-- Scorching Cloak 134634
	self:assertEquals(DestroyusPro.isItemDisenchantable(134634, 1, false),
	                  false, "Scorching Cloak 134634")
	-- Monastic Ring of Stoicism  88638  UniqueId
	self:assertEquals(DestroyusPro.isItemDisenchantable(88638, 1, false),
	                  false, "Monastic Ring of Stoicism  88638  UniqueId")
	-- Necropolis Breastplate 134729
	self:assertEquals(DestroyusPro.isItemDisenchantable(36397, 1, false),
	                  false, "Necropolis Breastplate 134729")
	-- Belt of the Blood Nova 50015 Quality epic 
	self:assertEquals(DestroyusPro.isItemDisenchantable(50015, 1, false),
	                  false, "Belt of the Blood Nova 50015 Quality epic") 
	-- Jacinth Circle 11969
	self:assertEquals(DestroyusPro.isItemDisenchantable(11969, 1, false),
	                  false, "Jacinth Circle 11969")
	-- Vindicator's Iron Legguards 26033 1 lvl disenchanting and armor
	self:assertEquals(DestroyusPro.isItemDisenchantable(26033, 1, false),
	                  true, "Vindicator's Iron Legguards 26033 1 lvl disenchanting and armor")
	GetProfessions = functionGetProfEnch

end

function ItemsTest.test_isItemDisenchantableTranslateItems(self)
	local functionGetProfEnch = GetProfessions
	GetProfessions = function() return 2,4,1,4,5,6,7 end 
	-- Emergency Hatchet 69223 One Hand Axe
	self:assertEquals(DestroyusPro.isItemDisenchantable(69223, 1, false),
	                  true, "Emergency Hatchet 69223 One Hand Axe") 
	-- Lupine Axe 1220 Two Hand Axe
	self:assertEquals(DestroyusPro.isItemDisenchantable(1220, 1, false),
	                  true, "Lupine Axe 1220 Two Hand Axe")
	-- Shipwreck Bow 52646 Bow
	self:assertEquals(DestroyusPro.isItemDisenchantable(52646, 1, false),
	                  true, "Shipwreck Bow 52646 Bow")
	-- Skorn's Rifle 3079 Gun 
	self:assertEquals(DestroyusPro.isItemDisenchantable(3079, 1, false),
	                  true, "Skorn's Rifle 3079 Gun")
	--Gnoll Punisher 1214 One Hand Mace
	self:assertEquals(DestroyusPro.isItemDisenchantable(1214, 1, false),
					  true, "Gnoll Punisher 1214 One Hand Mace")
	-- Codex Breaker 62878 Two Hand Mace 
	self:assertEquals(DestroyusPro.isItemDisenchantable(62878, 1, false),
					  true, "Codex Breaker 62878 Two Hand Mace")
	-- Stunted Tree Root 57214 Polearms
	self:assertEquals(DestroyusPro.isItemDisenchantable(57214, 1, false),
					  true, "Stunted Tree Root 57214 Polearms")
	-- Sunwell Blade 22783 One Hand Sword
	self:assertEquals(DestroyusPro.isItemDisenchantable(22783, 1, false),
					  true, "Sunwell Blade 22783 One Hand Sword")
	-- Brashclaw's Skewer 2204 Two hand Sword
	self:assertEquals(DestroyusPro.isItemDisenchantable(2204, 1, false),
					  true, "Brashclaw's Skewer 2204 Two hand Sword")
	-- Gnarled Hermit's Staff 1539 Staff
	self:assertEquals(DestroyusPro.isItemDisenchantable(1539, 1, false),
			          true, "Gnarled Hermit's Staff 1539 Staff")
	-- Filthy Paw  58978 Fist Weapon
	self:assertEquals(DestroyusPro.isItemDisenchantable(58978, 1, false),
	                  true, "Filthy Paw  58978 Fist Weapon")
	-- Venom Web Fang  899 Dagger
	self:assertEquals(DestroyusPro.isItemDisenchantable(899, 1, false),
	                  true, "Venom Web Fang  899 Dagger")
	-- Shrewd Crossbow  59582 Xbow 
	self:assertEquals(DestroyusPro.isItemDisenchantable(59582, 1, false),
	                  true, "Shrewd Crossbow  59582 Xbow")
	-- Brazie's Plant Light 65689 Wand
	self:assertEquals(DestroyusPro.isItemDisenchantable(65689, 1, false),
	                  true, "Brazie's Plant Light 65689 Wand")
	_G["DestroyusProItemQualityValue"] = DestroyusPro.strToNumQuality["rare"]
	-- Glaive of the Fallen 141604 Warglaives
	self:assertEquals(DestroyusPro.isItemDisenchantable(141604, 1, false), 
	                  true, "Glaive of the Fallen 141604 Warglaives")
	_G["DestroyusProItemQualityValue"] = DestroyusPro.strToNumQuality["uncommon"]
	GetProfessions = functionGetProfEnch
end

function ItemsTest.test_isItemDisenchantableTranslateArmor(self)
	local functionGetProfEnch = GetProfessions
	GetProfessions = function() return 2,4,1,4,5,6,7 end 
	-- Blue Overalls 6263 Cloth Armor
	self:assertEquals(DestroyusPro.isItemDisenchantable(6263, 1, false),
					  true, "Blue Overalls 6263 Cloth Armor")
	-- Dark Leather Tunic 2317 Leather Armor
	self:assertEquals(DestroyusPro.isItemDisenchantable(2317, 1, false),
					  true, "Dark Leather Tunic 2317 Leather Armor")
	-- Ironforge Breastplate 6731 Mail Armor
	self:assertEquals(DestroyusPro.isItemDisenchantable(6731, 1, false),
					  true, "Ironforge Breastplate 6731 Mail Armor")
	-- Stabilized Incendicite Legguards 59066 Plate
	self:assertEquals(DestroyusPro.isItemDisenchantable(59066, 1, false),
	                  true, "Stabilized Incendicite Legguards 59066 Plate")
	-- Wheel of the Lost Hope 24334 Shield
	self:assertEquals(DestroyusPro.isItemDisenchantable(24334, 1, false),
	                  true, "Wheel of the Lost Hope 24334 Shield")
	-- Greenweave Cloak 9770 back
	self:assertEquals(DestroyusPro.isItemDisenchantable(9770, 1, false),
	                  true, "Greenweave Cloak 9770 back")
	GetProfessions = functionGetProfEnch
end

function ItemsTest.test_isItemDisenchantableTranslateMiscellaneous(self)
	local functionGetProfEnch = GetProfessions
	GetProfessions = function() return 2,4,1,4,5,6,7 end 
	-- Malachite Pendant 25438 Neck
	self:assertEquals(DestroyusPro.isItemDisenchantable(25438, 1, false),
	                  true, "Malachite Pendant 25438 Neck")
	-- Windrunner Band 62892 Ring
	self:assertEquals(DestroyusPro.isItemDisenchantable(62892, 1, false),
	                  true, "Windrunner Band 62892 Ring")
	-- Hypnotist's Watch 25786 Trinket 225 ILVL
	self:assertEquals(DestroyusPro.isItemDisenchantable(25786, 1, false),
					  false, "Hypnotist's Watch 25786 Trinket 225 ILVL")
	-- Glowing Murloc Eye 52652 Off Hand
	self:assertEquals(DestroyusPro.isItemDisenchantable(52652, 1, false),
					  true, "Glowing Murloc Eye 52652 Off Hand")
	GetProfessions = functionGetProfEnch
end

function ItemsTest.test_isNotUniqueItem(self) 
	-- Monastic Ring of Stoicism  88638  UniqueId
	self:assertEquals(DestroyusPro.isNotUniqueItem(88638, DestroyusPro.itemsIdNotForDisenchant),
	                  false, "Monastic Ring of Stoicism  88638  UniqueId")
	self:assertEquals(DestroyusPro.isNotUniqueItem(38289, DestroyusPro.itemsIdNotForDisenchant),
	                  false, "Item should be unqiue!") 
	self:assertEquals(DestroyusPro.isNotUniqueItem(55290, DestroyusPro.itemsIdNotForDisenchant),
	                  true, "Item should not be unique!")
end

function ItemsTest.setUp(self)
	self.functionProfInfo = GetProfessionInfo
	GetProfessionInfo = GetProfessionInfoApi
	self.functionGetProf = GetProfessions
	GetProfessions = function() return 1,2,3,4,5,6,7 end
end

function ItemsTest.tearDown(self)
	GetProfessionInfo = self.functionProfInfo
	GetProfessions = self.functionGetProf
end
	
function RunTest()
	utr = UnitTestRunner()
	utr:with(HerbsAndOreTest())
	utr:with(ItemsTest())
	utr:run()
end